@@ -1,26 +1,20 @@
-/*
- * Copyright (c) 2003, 2004 Roger Seguin <roger_seguin@msn.com>
+/* $Id: main.c 4593 2008-04-16 17:38:56Z landry $
+ * 
+ * Copyright (c) 2003-2004 Roger Seguin <roger_seguin@msn.com>
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Library General Public License for more details.
  *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #include "config_gui.h"
@@ -40,6 +34,7 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <memory.h>
+#include <math.h>
 #include <string.h>
 #include <inttypes.h>
 #include <string.h>
@@ -86,7 +81,7 @@ typedef enum monitor_bar_order_t {
 typedef struct param_t {
     /* Configurable parameters */
     char            acDevice[64];
-#if  !defined(__NetBSD__)
+#if  !defined(__NetBSD__) && !defined(__OpenBSD__)
     dev_t           st_rdev;
 #endif
     int             fTitleDisplayed;
@@ -141,6 +136,8 @@ typedef struct diskperf_t {
 
 	/**************************************************************/
 
+static int timerNeedsUpdate = 0;
+
 static int DisplayPerf (struct diskperf_t *p_poPlugin)
  /* Get the last disk perfomance data, compute the statistics and update
     the panel-docked monitor bars */
@@ -155,15 +152,16 @@ static int DisplayPerf (struct diskperf_t *p_poPlugin)
     const double    K = 1.0 * 1000 * 1000 * 1000 / 1024 / 1024;
     /* bytes/ns --> MB/s */
     double          arPerf[NMONITORS], arBusy[NMONITORS], *prData, *pr;
-    char            acToolTips[128];
+    char            acToolTips[256];
     int             status, i;
 
     if (!s_poToolTips)
 	s_poToolTips = gtk_tooltips_new ();
 
+    rbytes = wbytes = iRBusy_ns = iWBusy_ns = -1;
     memset (&oPerf, 0, sizeof (oPerf));
     oPerf.qlen = -1;
-#if defined (__NetBSD__)
+#if defined (__NetBSD__) || defined(__OpenBSD__)
     status = DevGetPerfData (poConf->acDevice, &oPerf);
 #else
     status = DevGetPerfData (&(poConf->st_rdev), &oPerf);
@@ -203,26 +201,26 @@ static int DisplayPerf (struct diskperf_t *p_poPlugin)
 	}
     }
 
-    sprintf (acToolTips, "%s\n"
+    snprintf (acToolTips, sizeof(acToolTips), "%s\n"
 	     "----------------\n"
 	     "I/O    (MB/s)\n"
-	     "  Read :%3u\n"
-	     "  Write :%3u\n"
-	     "  Total :%3u\n"
+	     "  Read :%3.2f\n"
+	     "  Write :%3.2f\n"
+	     "  Total :%3.2f\n"
 	     "Busy time (%c)\n"
 	     "  Read : %3d\n"
 	     "  Write : %3d\n"
 	     "  Total : %3d",
 	     poConf->acTitle,
-	     (unsigned int) (arPerf[R_DATA] + 0.5),
-	     (unsigned int) (arPerf[W_DATA] + 0.5),
-	     (unsigned int) (arPerf[RW_DATA] + 0.5),
+	     arPerf[R_DATA],
+	     arPerf[W_DATA],
+	     arPerf[RW_DATA],
 	     '%',
 	     SEPARATE_BUSY_TIMES && (oPerf.qlen >= 0) ?
-	     (int) (arBusy[R_DATA] + 0.5) : -1,
+	     (int) round(arBusy[R_DATA]) : -1,
 	     SEPARATE_BUSY_TIMES && (oPerf.qlen >= 0) ?
-	     (int) (arBusy[W_DATA] + 0.5) : -1,
-	     (oPerf.qlen >= 0) ? (int) (arBusy[RW_DATA] + 0.5) : -1);
+	     (int) round(arBusy[W_DATA]) : -1,
+	     (oPerf.qlen >= 0) ? (int) round(arBusy[RW_DATA]) : -1);
     gtk_tooltips_set_tip (s_poToolTips, GTK_WIDGET (poMonitor->wEventBox),
 			  acToolTips, 0);
 
@@ -272,6 +270,12 @@ static gboolean SetTimer (void *p_pvPlugin)
     struct param_t *poConf = &(poPlugin->oConf.oParam);
 
     DisplayPerf (poPlugin);
+    
+    if (timerNeedsUpdate) {
+        g_source_remove (poPlugin->iTimerId);
+        poPlugin->iTimerId = 0;
+        timerNeedsUpdate = 0;
+    }
 
     if (!poPlugin->iTimerId)
         poPlugin->iTimerId = g_timeout_add (poConf->iPeriod_ms,
@@ -398,7 +402,7 @@ static diskperf_t *diskperf_create_control (XfcePanelPlugin *plugin)
     struct diskperf_t *poPlugin;
     struct param_t *poConf;
     struct monitor_t *poMonitor;
-#if !defined(__NetBSD__)
+#if !defined(__NetBSD__) && !defined(__OpenBSD__)
     struct stat     oStat;
     int             status;
 #endif
@@ -410,14 +414,14 @@ static diskperf_t *diskperf_create_control (XfcePanelPlugin *plugin)
 
     poPlugin->plugin = plugin;
     
-#if defined(__NetBSD__)
-    strcpy (poConf->acDevice, "wd0");
-    strcpy (poConf->acTitle, "wd0");
+#if defined(__NetBSD__) || defined(__OpenBSD__)
+    strncpy (poConf->acDevice, "wd0", 64);
+    strncpy (poConf->acTitle, "wd0", 16);
 #else
-    strcpy (poConf->acDevice, "/dev/hda");
+    strncpy (poConf->acDevice, "/dev/sda", 64);
     status = stat (poConf->acDevice, &oStat);
     poConf->st_rdev = (status == -1 ? 0 : oStat.st_rdev);
-    strcpy (poConf->acTitle, "hda");
+    strncpy (poConf->acTitle, "sda", 16);
 #endif
 
     poConf->fTitleDisplayed = 1;
@@ -481,7 +485,7 @@ static void diskperf_read_config (XfcePanelPlugin *plugin,
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     struct monitor_t *poMonitor = &(poPlugin->oMonitor);
     Widget_t       *pw2ndBar = poPlugin->oMonitor.awProgressBar + 1;
-#if !defined(__NetBSD__)
+#if !defined(__NetBSD__) && !defined(__OpenBSD__)
     struct stat     oStat;
     int             status;
 #endif
@@ -498,7 +502,7 @@ static void diskperf_read_config (XfcePanelPlugin *plugin,
     if ((value = xfce_rc_read_entry (rc, (CONF_DEVICE), NULL))) {
         memset (poConf->acDevice, 0, sizeof (poConf->acDevice));
         strncpy (poConf->acDevice, value, sizeof (poConf->acDevice) - 1);
-#if !defined(__NetBSD__)
+#if !defined(__NetBSD__) && !defined(__OpenBSD__)
         status = stat (poConf->acDevice, &oStat);
         poConf->st_rdev = (status == -1 ? 0 : oStat.st_rdev);
 #endif
@@ -595,17 +599,17 @@ static void diskperf_write_config (XfcePanelPlugin *plugin,
                              poConf->eMonitorBarOrder);
 
     poColor = poConf->aoColor + R_DATA;
-    sprintf (acBuffer, acColorFormat,
+    snprintf (acBuffer, 16, acColorFormat,
 	     poColor->red >> 8, poColor->green >> 8, poColor->blue >> 8);
     xfce_rc_write_entry (rc, CONF_READ_COLOR, acBuffer);
 
     poColor = poConf->aoColor + W_DATA;
-    sprintf (acBuffer, acColorFormat,
+    snprintf (acBuffer, 16, acColorFormat,
 	     poColor->red >> 8, poColor->green >> 8, poColor->blue >> 8);
     xfce_rc_write_entry (rc, CONF_WRITE_COLOR, acBuffer);
 
     poColor = poConf->aoColor + RW_DATA;
-    sprintf (acBuffer, acColorFormat,
+    snprintf (acBuffer, 16, acColorFormat,
 	     poColor->red >> 8, poColor->green >> 8, poColor->blue >> 8);
     xfce_rc_write_entry (rc, CONF_READ_WRITE_COLOR, acBuffer);
 
@@ -621,7 +625,7 @@ static void SetDevice (Widget_t p_wTF, void *p_pvPlugin)
     struct diskperf_t *poPlugin = (diskperf_t *) p_pvPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     const char     *pcDevice = gtk_entry_get_text (GTK_ENTRY (p_wTF));
-#if !defined(__NetBSD__)
+#if !defined(__NetBSD__) && !defined(__OpenBSD__)
     struct stat     oStat;
     int             status;
 
@@ -753,15 +757,14 @@ static void SetXferRate (Widget_t p_wTF, void *p_pvPlugin)
     struct diskperf_t *poPlugin = (diskperf_t *) p_pvPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     const char     *pcXferRate = gtk_entry_get_text (GTK_ENTRY (p_wTF));
-    float           r;
-
-    poConf->iMaxXferMBperSec = atoi (pcXferRate);
 
     /* Make it a multiple of 5 MB/s */
-    r = (double) poConf->iMaxXferMBperSec / 5;
-    poConf->iMaxXferMBperSec = 5 * (int) (r + 0.5);
+    poConf->iMaxXferMBperSec = 5 * round((double) atoi(pcXferRate) / 5);
     if (poConf->iMaxXferMBperSec > 995)
 	poConf->iMaxXferMBperSec = 995;
+    else if (poConf->iMaxXferMBperSec < 5)
+	poConf->iMaxXferMBperSec = 5;
+    DBG("XferRate rounded to %dMb/s\n", poConf->iMaxXferMBperSec);
 }				/* SetXferRate() */
 
 	/**************************************************************/
@@ -773,14 +776,11 @@ static void SetPeriod (Widget_t p_wSc, void *p_pvPlugin)
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     float           r;
 
+    timerNeedsUpdate = 1;
     TRACE ("SetPeriod()\n");
     r = gtk_spin_button_get_value (GTK_SPIN_BUTTON (p_wSc));
-    poConf->iPeriod_ms = (r * 1000) + 0.5;	/* rounded */
-
-    /* Make it a multiple of 50 ms */
-    r = (double) poConf->iPeriod_ms / 50;
-    poConf->iPeriod_ms = 50 * (int) (r + 0.5);
-
+    poConf->iPeriod_ms = round(r * 1000);
+    DBG("Update period rounded to %dms\n", poConf->iPeriod_ms);
 }				/* SetPeriod() */
 
 	/**************************************************************/
@@ -909,7 +909,7 @@ static void diskperf_create_options (XfcePanelPlugin *plugin,
 	/* Plugin API */
 	/* Create/pop up the configuration/options GUI */
 {
-    GtkWidget *dlg, *header, *vbox;
+    GtkWidget *dlg, *vbox;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     struct gui_t   *poGUI = &(poPlugin->oConf.oGUI);
     char            acBuffer[16];
@@ -923,24 +923,17 @@ static void diskperf_create_options (XfcePanelPlugin *plugin,
 
     xfce_panel_plugin_block_menu (plugin);
     
-    dlg = gtk_dialog_new_with_buttons (_("Configuration"), 
-                GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
-                GTK_DIALOG_DESTROY_WITH_PARENT |
-                GTK_DIALOG_NO_SEPARATOR,
-                GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
-                NULL);
+    dlg = xfce_titled_dialog_new_with_buttons (_("Disk Performance Monitor"),
+                                                GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
+                                                GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
+                                                GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
+                                                NULL);
     
-    g_signal_connect (dlg, "response", G_CALLBACK (diskperf_dialog_response),
-                      poPlugin);
+    g_signal_connect (G_OBJECT (dlg), "response",
+                      G_CALLBACK (diskperf_dialog_response), poPlugin);
 
-    gtk_container_set_border_width (GTK_CONTAINER (dlg), 2);
-    
-    header = xfce_create_header (NULL, _("Disk Performance Monitor"));
-    gtk_widget_set_size_request (GTK_BIN (header)->child, -1, 32);
-    gtk_container_set_border_width (GTK_CONTAINER (header), BORDER - 2);
-    gtk_widget_show (header);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), header,
-                        FALSE, TRUE, 0);
+    gtk_window_set_position (GTK_WINDOW (dlg), GTK_WIN_POS_CENTER);
+    gtk_window_set_icon_name (GTK_WINDOW (dlg), "xfce4-settings");
     
     vbox = gtk_vbox_new(FALSE, BORDER);
     gtk_container_set_border_width (GTK_CONTAINER (vbox), BORDER - 2);
@@ -950,7 +943,7 @@ static void diskperf_create_options (XfcePanelPlugin *plugin,
     
     poPlugin->oConf.wTopLevel = dlg;
 
-    (void) CreateConfigGUI (vbox, poGUI);
+    CreateConfigGUI (vbox, poGUI);
 
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (poGUI->wTB_Title),
 				  poConf->fTitleDisplayed);
@@ -1000,7 +993,7 @@ static void diskperf_create_options (XfcePanelPlugin *plugin,
     g_signal_connect (GTK_WIDGET (poGUI->wTF_Title), "activate",
 		      G_CALLBACK (SetLabel), poPlugin);
 
-    sprintf (acBuffer, "%d", poConf->iMaxXferMBperSec);
+    snprintf (acBuffer, 16, "%d", poConf->iMaxXferMBperSec);
     gtk_entry_set_text (GTK_ENTRY (poGUI->wTF_MaxXfer), acBuffer);
     g_signal_connect (GTK_WIDGET (poGUI->wTF_MaxXfer), "activate",
 		      G_CALLBACK (SetXferRate), poPlugin);
@@ -1033,10 +1026,6 @@ static void diskperf_create_options (XfcePanelPlugin *plugin,
 	g_signal_connect (GTK_WIDGET (*(apwColorPB[i])), "clicked",
 			  G_CALLBACK (ChooseColor), poPlugin);
     }
-
-    g_signal_connect (GTK_WIDGET (poGUI->wPB_About), "clicked",
-		      G_CALLBACK (About), 0);
-		      
 		      
     gtk_widget_show (dlg);
 }				/* diskperf_create_options() */
@@ -1055,11 +1044,13 @@ static gboolean diskperf_set_size (XfcePanelPlugin *plugin, int p_size,
     if (xfce_panel_plugin_get_orientation (plugin) == 
             GTK_ORIENTATION_HORIZONTAL) {
 	size1 = BORDER;
-        size2 = p_size - 4;
+        size2 = -1;
+        gtk_widget_set_size_request (GTK_WIDGET (plugin), -1, p_size);
     }
     else {
-	size1 = p_size - 4;
+	size1 = -1;
         size2 = BORDER;
+        gtk_widget_set_size_request (GTK_WIDGET (plugin), p_size, -1);
     }
     for (i = 0; i < 2; i++) {
 	pwBar = poPlugin->oMonitor.awProgressBar + i;
@@ -1096,8 +1087,6 @@ static void diskperf_construct (XfcePanelPlugin *plugin)
 {
     diskperf_t *diskperf = diskperf_create_control (plugin);
 
-    diskperf_read_config (plugin, diskperf);
-
     g_signal_connect (plugin, "free-data", G_CALLBACK (diskperf_free), 
                       diskperf);
 
@@ -1121,76 +1110,10 @@ static void diskperf_construct (XfcePanelPlugin *plugin)
 
     CreateMonitorBars (diskperf, xfce_panel_plugin_get_orientation (plugin));
     
+    diskperf_read_config (plugin, diskperf);
+    DevPerfInit();
+    
     SetTimer (diskperf);
 }
 
 XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL (diskperf_construct);
-
-
-	/**************************************************************/
-/*
-$Log: main.c,v $
-Revision 1.7  2004/08/25 10:08:40  rogerms
-DiskPerf 1.5
-
-Revision 1.14  2004/08/25 08:51:02  RogerSeguin
-MT support and About... dialog box added
-
-Revision 1.6  2003/11/11 12:40:34  rogerms
-Release 1.4
-
-Revision 1.13  2003/11/11 12:12:42  RogerSeguin
-Force to single "total busy time" monitor when platform doesn't provide separate read and write data (e.g. NetBSD)
-
-Revision 1.5  2003/11/04 10:26:13  rogerms
-DiskPerf 1.3
-
-Revision 1.12  2003/11/04 10:16:36  RogerSeguin
-Got rid of Microsoft ^M
-
-Revision 1.11  2003/11/04 09:43:36  RogerSeguin
-Added busy time monitoring for Linux
-
-Revision 1.4  2003/11/02 06:57:50  rogerms
-Release 1.2
-
-Revision 1.10  2003/11/02 06:18:33  RogerSeguin
-Added busy time in tooltips for Linux 2.4 and 2.6
-
-Revision 1.9  2003/10/24 11:16:20  RogerSeguin
-Different scalable fonts with Mandrake 9.2 ==> diffent tooltips string spacing
-
-Revision 1.3  2003/10/18 23:02:58  rogerms
-DiskPerf release 1.1
-
-Revision 1.8  2003/10/18 06:56:50  RogerSeguin
-Integration of Benedikt Meurer's work on NetBSD port
-
-Revision 1.7  2003/10/16 13:07:42  RogerSeguin
-Kernel 2.6 support
-
-Revision 1.2  2003/10/16 18:48:39  benny
-Added support for NetBSD.
-
-Revision 1.1.1.1  2003/10/07 03:39:25  rogerms
-Initial release - v1.0
-
-Revision 1.6  2003/10/02 04:16:07  RogerSeguin
-Compute using rbytes/wbytes instead of rsect/wsect
-
-Revision 1.5  2003/09/25 12:24:11  RogerSeguin
-Implemented some error processing
-
-Revision 1.4  2003/09/25 09:32:13  RogerSeguin
-Added color configuration
-
-Revision 1.3  2003/09/24 10:56:36  RogerSeguin
-Now swapping the monitor bars is possible
-
-Revision 1.2  2003/09/23 15:17:01  RogerSeguin
-Now supports panel orientation
-
-Revision 1.1  2003/09/22 02:25:35  RogerSeguin
-Initial revision
-
-*/
